//  This file is part of the Mussa source distribution.
//  http://mussa.caltech.edu/
//  Contact author: Tristan  De Buysscher, tristan@caltech.edu

// This program and all associated source code files are Copyright (C) 2005
// the California Institute of Technology, Pasadena, CA, 91125 USA.  It is
// under the GNU Public License; please see the included LICENSE.txt
// file for more information, or contact Tristan directly.



#include "alg/nway_paths.hpp"
#include <math.h>
#include <iostream>
using namespace std;

// vars that will be availble to entropy function when in its nway class
//vector<char *> c_sequences;
//int window = 5, seq_num;


void
NwayPaths::setup_ent(double new_entropy_thres, vector<Sequence> some_seqs)
{
  ent_thres = new_entropy_thres;

  c_sequences.clear();
  for (vector<string>::size_type i = 0; i < some_seqs.size(); i++)
    c_sequences.push_back((char *)some_seqs[i].get_sequence().c_str());
}


double
NwayPaths::path_entropy(vector<int> path)
{
  int bp_occurences[5];
  double bp_prob, frac_ent, avg_entropy;
  vector<double> entropies;
  int i, i2;
  int window = win_size;

  for(i = 0; i < window; i++)     // loop thru all bp positions
  {
    for(i2 = 0; i2 < 5; i2++)     // clear old occurences
      bp_occurences[i2] = 0;

    //loop thru all sequences at pos i
    for(vector<int>::size_type seq_i = 0; seq_i != path.size(); seq_i++)
    {
      if (path[seq_i] > -1)
      {
        switch (c_sequences[seq_i][i+path[seq_i]])
        {
          case 'A':
            ++bp_occurences[0];
            break;
          case 'T':
            ++bp_occurences[1];
            break;
          case 'G':
            ++bp_occurences[2];
            break;
          case 'C':
            ++bp_occurences[3];
            break;
          case 'N':
            ++bp_occurences[4];
            break;
          default:
            cout << "error, nonAGCTN on seq: " << seq_i << " at index: ";
            cout << i+path[seq_i] << endl;
        }
      }
      else
      {
        switch (c_sequences[seq_i][-1 * (i + path[seq_i] - window + 1) ])
        {
          case 'A':
            ++bp_occurences[1];
            break;
          case 'T':
            ++bp_occurences[0];
            break;
          case 'G':
            ++bp_occurences[3];
            break;
          case 'C':
            ++bp_occurences[2];
            break;
          case 'N':
            ++bp_occurences[4];
            break;
          default:
            cout << "error, nonAGCTN on seq: " << seq_i << " at index: ";
            cout << (i + path[seq_i] - window + 1) << endl;
        }
      }
    }
    
    entropies.push_back(0.000);
    for(i2 = 0; i2 < 5; i2++)    //calculate entropies
    {
      bp_prob = (double) bp_occurences[i2] / (double) path.size();
      //cout << bp_prob << "::";
      if (bp_prob == 0.0000)
        frac_ent = 0.0000;
      else
        frac_ent = bp_prob * ( log10(bp_prob) / log10(2.0) );
      //cout << frac_ent << "  ";
      entropies[i] += frac_ent;
    }
    //cout << endl;
  }

  avg_entropy = 0;
  for(i = 0; i < window; i++)
  {
    avg_entropy += entropies[i];
    //cout  << entropies[i] << endl;
  }

  avg_entropy = -1.00 * (avg_entropy / (double) window);
  //cout << "average entropy: " << avg_entropy << endl;
  return avg_entropy;
}

void
NwayPaths::entropy_path_search(vector<vector<FLPs> > all_comparisons)
{
  vector<int> path;
  double avg_entropy;
  int win_i, sp_depth, window_num;
  unsigned int sp_i;
  bool some_matches, still_paths, not_advanced;
  list<int> new_nodes, trans_check_nodes;
  vector<list<int> > all_matches;
  vector<list<int>::iterator> sp_nodes, sp_nodes_end;
  list<int>::iterator debug_i;


  pathz.clear();
  window_num = all_comparisons[0][1].size();
  cout << window_num << endl;    // just a sanity check

  sp_nodes.reserve(all_comparisons.size());
  sp_nodes_end.reserve(all_comparisons.size());

  // loop thru all windows in first species
  for (win_i = 0; win_i < window_num; win_i++)
  {
    // first we see if the first seq has matches to all other seqs at this win
    some_matches = true;
    sp_i = 1;
    all_matches.clear();
    while ( (sp_i < all_comparisons.size()) && (some_matches) )
    {
      new_nodes.clear();
      new_nodes = all_comparisons[0][sp_i].match_locations(win_i);
      if (new_nodes.empty())
        some_matches = false;

      all_matches.push_back(new_nodes);
      sp_i++;
    }
    //cout << "fee\n";

    // if 1st seq does match to all others, make all possible paths
    // out of all these matches
    if (some_matches)
    {
      sp_nodes.clear();
      sp_nodes_end.clear();
      // set each species list of matches to beginning
      for (sp_i = 0; sp_i < all_comparisons.size()-1; sp_i++)
      {
        sp_nodes.push_back(all_matches[sp_i].begin());
        sp_nodes_end.push_back(all_matches[sp_i].end());
      }

      still_paths = true;
      //cout << "fie\n";
      while (still_paths) 
      {
        // add path that each species iterator is pointing to
        path.clear();
        path.push_back(win_i);
	
        //cout << win_i;
        for (sp_i = 0; sp_i < all_comparisons.size()-1; sp_i++)
        {
          //cout  << ", " << *(sp_nodes[sp_i]);
          path.push_back(*(sp_nodes[sp_i]));
        }
        //cout << endl;
 
        // check entropy <---------------------------------------------------
        avg_entropy = path_entropy(path);
        if (avg_entropy <= ent_thres)
          pathz.push_back(ConservedPath(win_size, avg_entropy, path));
        
        // now advance the right iterator
        not_advanced = true;
        sp_depth = all_comparisons.size()- 2;  // this roves up & down species list
        while ( (not_advanced) && (sp_depth != -1) )
        {
          //cout << "foe\n";
          //cout << sp_depth << ".." << *(sp_nodes[sp_depth]) << endl;
          (sp_nodes[sp_depth])++;   // advance iter at current depth
          //cout << sp_depth << ".." << *(sp_nodes[sp_depth]) << endl;
          
          // if we reached the end of the list, reset iter & go up one depth
          if (sp_nodes[sp_depth] == sp_nodes_end[sp_depth])
          {
            //cout << "fum\n";
            sp_nodes[sp_depth] = all_matches[sp_depth].begin();
            sp_depth--;
            //cout << "depth = " << sp_depth << endl;
          }
          else
            not_advanced = false;
        }
        
        if (sp_depth == -1)   // jumped up to first species, all paths searched
          still_paths = false;
        else                 // otherwise just reset to max depth and continue
          sp_depth = all_comparisons.size() - 2;
      }
    }
  }
}


//initial coding testing 
/*
int main(int argc, char **argv) 
{
  vector<string> sequences;
  vector<int> i_starts;
  double avg_entropy; 
  int seq_i;

  sequences.clear();
  sequences.push_back("AAAAA");
  sequences.push_back("AAAAT");
  sequences.push_back("AATTG");
  sequences.push_back("ATTGC");

  seq_num = 4;

  c_sequences.clear();
  i_starts.clear();
  for(seq_i = 0; seq_i < seq_num; seq_i++)   //loop thru all sequences at pos i
  {
    c_sequences.push_back((char *)sequences[seq_i].c_str());
    i_starts.push_back(0);
  }

  avg_entropy = Window_Entropy(i_starts);

  cout << "average entropy: " << avg_entropy << endl;
}
*/
