#define BOOST_AUTO_TEST_MAIN
#include <boost/test/auto_unit_test.hpp>

#include "alg/annotation_colors.hpp"
#include "alg/color.hpp"

BOOST_AUTO_TEST_CASE( simple_annot_colors )
{
  Color black(0.0, 0.0, 0.0);
  Color white(1.0, 1.0, 1.0);
  Color red(1.0, 0.0, 0.0);

  AnnotationColors ac;

  ac.setColor(white);
  BOOST_CHECK_EQUAL( ac.color(), white);

  ac.appendTypeColor("bleem", black);
  BOOST_CHECK_EQUAL( ac.typeColor("bleem"), black );
  // FIXME: it'd be nice if something like this worked?
  //BOOST_CHECK_RAISES( ac.typeColor("not found"), not_found );
  ac.appendInstanceColor("bleem", "a", red);
  BOOST_CHECK_EQUAL( ac.instanceColor("bleem", "a"), red);
 
  BOOST_CHECK_EQUAL( ac.lookup("bleem", "a"), red );
  BOOST_CHECK_EQUAL( ac.lookup("bleem", "foo"), black );
  BOOST_CHECK_EQUAL( ac.lookup("venchent", "a"), white );

  /*
  // Removed as we're phasing out the annot and annotation colors classes
  annot a;
  a.begin = 30;
  a.end = 45;
  a.type = "bleem";
  a.name = "a";
  
  BOOST_CHECK_EQUAL( ac.lookup(a), red );
  a.name = "b";
  BOOST_CHECK_EQUAL( ac.lookup(a), black );
  a.type = "venchent";
  BOOST_CHECK_EQUAL( ac.lookup(a), white );
  */
  
  ac.clear();
  BOOST_CHECK_EQUAL( ac.lookup("bleem", "a"), black);
}

// can we assign a color?
BOOST_AUTO_TEST_CASE( annotation_color_assignment )
{
  Color black(0.0, 0.0, 0.0);
  Color white(1.0, 1.0, 1.0);
  Color red(1.0, 0.0, 0.0);

  AnnotationColors ac1;
  ac1.setColor(white);
  ac1.appendTypeColor("bleem", black);
  ac1.appendInstanceColor("bleem", "a", red);

  AnnotationColors ac2;
  BOOST_CHECK_EQUAL( ac2.color(), black );
  ac2 = ac1;
  BOOST_CHECK_EQUAL( ac2.color(), white );
  BOOST_CHECK_EQUAL( ac2.typeColor("bleem"), ac1.typeColor("bleem") );
  BOOST_CHECK_EQUAL( ac2.lookup("bleem","a"), ac1.lookup("bleem","a") );
}
