#ifndef _SEQUENCE_DESCRIPTION_H
#define _SEQUENCE_DESCRIPTION_H

#include <boost/shared_ptr.hpp>

#include <string>

#include <QString>
#include <QLabel>
#include <QLineEdit>
#include <QFrame>
#include <QVBoxLayout>

#include "alg/glsequence.hpp"

class SequenceDescription : public QFrame
{
  Q_OBJECT 

public:
  SequenceDescription(QWidget *parent=0);
  SequenceDescription(boost::shared_ptr<GlSequence>, QWidget *);
  
  void setName(std::string name);
  boost::shared_ptr<GlSequence> glsequence();
  std::string name() const; 
  int length() const;
  int position() const;

public slots:
  //! Display the passed GlSequence information
  void setGlSequence(boost::shared_ptr<GlSequence>);
  //! set the name for our sequence
  void setName(const QString& name);
  void setPosition(int pos);

signals:
  void glsequenceChanged(boost::shared_ptr<GlSequence>);
  void nameChanged(const QString& name);
  void positionChanged(int pos);

private:
  QVBoxLayout *layout;
  QLineEdit *name_label;
  QLabel *length_label;
  QLabel *position_label;

  boost::shared_ptr<GlSequence> glsequence_;
  int pos;

  //! finish initializing GUI widgets
  void setupWidgets();
  //! format the length and set the QLabel
  void setLength(int length);
};
#endif
